/* Placed in the public domain in 2000 by Sam Trenholme */
/* JessicaStrings  -- This is my library for handling string manuipulations
   in a secure manner.  In order to make things secure, we have a more
   complicated structure for the string.  In addition to a pointer to the
   beginning of the actual character string, we have an indiction of how 
   long the string surrently is (ascii nulls should be allowed in strings.
   period), how long the string is allowed to be (it will truncate any attempt
   to overflow the string), and how long each character in the string is
   (can you say unicode.  I thought so) */

typedef struct {
   char *string;   /* Actual physical string */
   int unit_size;  /* The size of a single character in the string */
   int unit_count; /* The length of the string, in units */
   int max_count;  /* The maximum allowable size of the string, also in
                      units */
   int encoding;   /* The type of language/encoding the string is in */
   int is_good;    /* This is checked to make sure the data structure is
                      sane */
   } JessicaString;

/* Some constants */
/* Sane number for is_good structure element */
#define JS_SANE_NUMBER 3124
/* Some encoding numbers */
/* Minimum encoding number */
#define JS_MIN_ENCODE 1
/* encoding values */
#define JS_BINARY 1
#define JS_US_ASCII 2
/* Max encoding number */
#define JS_MAX_ENCODE 2

/* Function Prototypes */

/* JessStrOS.c functions */
void *js_alloc(int unit_count,int unit_size);
/* int js_dealloc((void *)pointer); */

/* JessStr.c functions */
JessicaString *js_create(int unit_count, int unit_size);
int js_set_encode(JessicaString *js, int encoding);
int js_destroy(JessicaString *object);
int js_issame(JessicaString *js1, JessicaString *js2);
int js_fgrep(JessicaString *exp, JessicaString *js);
int js_insert(JessicaString *toinsert, JessicaString *target, int where);

