/* Public domain code by Sam Trenholme 2000 */
/* Routines that interface between the string library and the underlying
   OS */

/* Headers for the underlying OS calls */
#include <stdlib.h>
#include <stdio.h>

/* Headers for the string routines */
#include "JessStr.h"

/* js_alloc: Allocate memory from the underlying OS.
   input: The unit count and unit size of memory to allocate
   output: An anonymous pointer to the allocated memory */

void *js_alloc(int unit_count, int unit_size) {
    void *data;
    data = (void *)malloc(unit_count * unit_size);
    if(data == NULL)
        return (void *)-1;
    return data;
    }

/* js_dealloc: Deallocate memory from the underlying OS
   input: A pointer to the memory we wish to free
   output: -1 on failure */

int js_dealloc(void *pointer) {
    free(pointer);
    return 0;
    }

/* js_show_string_stdout: Display the contents of a given JessicaString
                          object on standard output
   input: Pointer to JessicaString object
   output: -1 on failure */
int js_show_string_stdout(JessicaString *js) {
    int counter = 0;

    if(js_has_sanity(js) < 0) 
        return -1;

    while(counter < js->unit_size * js->unit_count) {
        putc(*(js->string + counter),stdout);
        counter++;
        }
    
    return 0;
    }

